#!/usr/bin/env bash
# ------------------------------------------------------------------------------
# This file profiles the provided binary on execution for the set platform.
# ------------------------------------------------------------------------------

# Source helpers for colored prints.
source helpers.sh

# The BASEDIR is the PWD of profile.sh, regardless of the callers location.
export BASEDIR=$(cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd)


# Check if PLATFORM is valid.
function platformCheck {
    IFS="|"
    options=("lumi${IFS}mn5")

    if [[ -z "${PLATFORM}" ]]; then
        fatal "No platform selected. Options: [ lumi | mn5 ]"
        exit $?
    elif [[ ! "${IFS}${options[*]}${IFS}" =~ "${IFS}${$PLATFORM}${IFS}" ]]; then
        fatal "Platform '${PLATFORM}' unknown. Options: [ lumi | mn5 ]"
        exit $?
    fi
}

# Profile binary on Lumi.
function profile_lumi {
    # Setup GPU and OpenMP aware hardware config.
    CPU_BIND="mask_cpu:7e000000000000,7e00000000000000"
    CPU_BIND="${CPU_BIND},7e0000,7e000000"
    CPU_BIND="${CPU_BIND},7e,7e00"
    CPU_BIND="${CPU_BIND},7e00000000,7e0000000000"

    export OMP_NUM_THREADS=1
    export MPICH_GPU_SUPPORT_ENABLED=1

    # Create folder for results.
    RESULTS=$BASEDIR/res-$1
    mkdir -p $RESULTS

    # Profile the provided binary.
    info "[$(date +'%H:%M:%S')]  Executing: $1"

    ROCMFILES="$RESULTS/rocprof.csv"
    srun --ntasks=1 --cpu-bind=${CPU_BIND} --output=$RESULTS/out.%j.%t \
            --error=$RESULTS/err.%j.%t --input=none \
            rocprof --hip-trace --roctx-trace -o $ROCMFILES \
            "$SELECT_GPU_NAME" $BASEDIR/../bin/$1
}

# Profile binary on MareNostrum 5.
function profile_mn5 {
    # TODO: Implement.
}

# Entry function.
function main {
    # Check if platform is set correctly.
    platformCheck

    # Check if provided binary exists.
    command -v $BASEDIR/../bin/$1 >/dev/null @>&1 || \
        fatal "Binary '$1' is not in bin folder."

    # Load modules.
    source $BASEDIR/../load_modules.sh

    # Profile provided binary on set platform.
    if [ "$PLATFORM" == "lumi" ]; then
        profile_lumi $1
        # sbatch profile_lumi.sh $1
    elif [ "$PLATFORM" == "mn5" ]; then
        profile_mn5 $1
    fi
}

main "$@"
